var RE = {};

RE.editor = document.getElementById('editor');

RE.callbackhandler = function () {
    return function () {
        window.android.callback(encodeURI(RE.getHtml()));
        RE.enabledEditingItems();
    }
}

RE.callback = function () {
    if (RE.timeoutProcess) {
        clearTimeout(RE.timeoutProcess);
    }

    RE.timeoutProcess = setTimeout(RE.callbackhandler(), 800);
}

RE.log = function (str) {
    window.android.log(str);
}

RE.setHtml = function (contents) {
    RE.log(decodeURIComponent(contents.replace(/\+/g, '%20')));
    RE.editor.innerHTML = decodeURIComponent(contents.replace(/\+/g, '%20'));
}

RE.getHtml = function () {
    return RE.editor.innerHTML;
}

RE.getText = function () {
    return RE.editor.innerText;
}

RE.setBaseTextColor = function (color) {
    RE.editor.style.color = color;
}

RE.setBaseFontSize = function (size) {
    RE.editor.style.fontSize = size;
}

RE.setPadding = function (left, top, right, bottom) {
    RE.editor.style.paddingLeft = left;
    RE.editor.style.paddingTop = top;
    RE.editor.style.paddingRight = right;
    RE.editor.style.paddingBottom = bottom;
}

RE.setBackgroundColor = function (color) {
    document.body.style.backgroundColor = color;
}

RE.setBackgroundImage = function (image) {
    RE.editor.style.backgroundImage = image;
}

RE.setWidth = function (size) {
    RE.editor.style.minWidth = size;
}

RE.setHeight = function (size) {
    RE.editor.style.height = size;
}

RE.setTextAlign = function (align) {
    RE.editor.style.textAlign = align;
}

RE.setVerticalAlign = function (align) {
    RE.editor.style.verticalAlign = align;
}

RE.setPlaceholder = function (placeholder) {
    RE.editor.setAttribute("placeholder", placeholder);
}

//api

RE.undo = function () {
    document.execCommand('undo', false, null);
}

RE.redo = function () {
    document.execCommand('redo', false, null);
}

RE.setBold = function () {
    document.execCommand('bold', false, null);
    RE.enabledEditingItems();
}

RE.setItalic = function () {
    document.execCommand('italic', false, null);
    RE.enabledEditingItems();
}

RE.setSubscript = function () {
    document.execCommand('subscript', false, null);
    RE.enabledEditingItems();
}

RE.setSuperscript = function () {
    document.execCommand('superscript', false, null);
    RE.enabledEditingItems();
}

RE.setStrikeThrough = function () {
    document.execCommand('strikeThrough', false, null);
    RE.enabledEditingItems();
}

RE.setUnderline = function () {
    document.execCommand('underline', false, null);
    RE.enabledEditingItems();
}

RE.setBullets = function () {
    document.execCommand('insertUnorderedList', false, null);
    RE.enabledEditingItems();
}

RE.setNumbers = function () {
    document.execCommand('insertOrderedList', false, null);
    RE.enabledEditingItems();
}

RE.setTextColor = function (color) {
    document.execCommand('foreColor', false, color);
}

RE.setTextBackgroundColor = function (color) {
    document.execCommand('hiliteColor', false, color);
}

RE.setFontSize = function (fontSize) {
    document.execCommand("fontSize", false, fontSize);
}

RE.setHeading = function (heading) {
    document.execCommand('formatBlock', false, '<h' + heading + '>');
    RE.enabledEditingItems();
}

RE.setIndent = function () {
    document.execCommand('indent', false, null);
}

RE.setOutdent = function () {
    document.execCommand('outdent', false, null);
}

RE.setJustifyLeft = function () {
    document.execCommand('justifyLeft', false, null);
    RE.enabledEditingItems();
}

RE.setJustifyCenter = function () {
    document.execCommand('justifyCenter', false, null);
    RE.enabledEditingItems();
}

RE.setJustifyRight = function () {
    document.execCommand('justifyRight', false, null);
    RE.enabledEditingItems();
}

RE.setBlockquote = function () {
    if (window.getSelection().getRangeAt(0).startContainer.nodeName == '#text') {
        document.execCommand('formatBlock', false, '<blockquote>');
        window.getSelection().focusNode.parentNode.className = 'code';
    }
    RE.enabledEditingItems();
}

RE.insertImage = function (url, alt) {
    var html = '<img src="' + url + '" alt="' + alt + '" />';
    RE.insertHTML(html);
}

RE.insertLocalProgressiveImage = function (source, alt, id) {
    var img = document.createElement('img');
    img.setAttribute("id", id);
    img.setAttribute("src", source);
    img.setAttribute("alt", alt);
    img.setAttribute("class", "localProgressiveImage");
    RE.insertHTML(img.outerHTML);
}

RE.replaceLocalProgressiveImageWithUrlImage = function (url, id) {
    var img = document.getElementById(id);
    img.setAttribute("src", url);
    img.className = 'urlImage'
    RE.callback();
}

RE.replaceLocalProgressiveVideoWithUrlVideo = function (url, id) {
    var video = document.getElementById(id);
    RE.log(id);
    video.outerHTML = '<video controls="controls" preload="metadata"><source src="' + url + '" type="video/mp4"></video><br>';
    RE.callback();
}

RE.insertVideo = function (url) {
    var html = '<video controls="controls" preload="metadata"><source src="' + url + '" type="video/mp4"></video><br>';
    RE.insertHTML(html);
}

RE.insertHTML = function (html) {
    document.execCommand('insertHTML', false, html);
}

RE.insertLink = function (url, title) {
    var sel = document.getSelection();
    if (sel.toString().length == 0) {
        document.execCommand("insertHTML", false, "<a href='" + url + "'>" + title + "</a>");
    } else if (sel.rangeCount) {
        var el = document.createElement("a");
        el.setAttribute("href", url);
        el.setAttribute("title", title);

        var range = sel.getRangeAt(0).cloneRange();
        range.surroundContents(el);
        sel.removeAllRanges();
        sel.addRange(range);
    }
    RE.callback();
}

RE.setTodo = function (text) {
    var html = '<input type="checkbox" name="' + text + '" value="' + text + '"/> &nbsp;';
    document.execCommand('insertHTML', false, html);
}

RE.removeFormat = function () {
    document.execCommand('removeFormat', false, null);
    var formatBlock = document.queryCommandValue('formatBlock');
    if (formatBlock.length > 0) {
        document.execCommand('formatBlock', false, 'p');
    }
}

//检测当前光标位置的样式
RE.enabledEditingItems = function (e) {
    var items = [];
    if (document.queryCommandState('bold')) {
        items.push('bold');
    }
    if (document.queryCommandState('italic')) {
        items.push('italic');
    }
    if (document.queryCommandState('subscript')) {
        items.push('subscript');
    }
    if (document.queryCommandState('superscript')) {
        items.push('superscript');
    }
    if (document.queryCommandState('strikeThrough')) {
        items.push('strikeThrough');
    }
    if (document.queryCommandState('underline')) {
        items.push('underline');
    }
    if (document.queryCommandState('insertOrderedList')) {
        items.push('orderedList');
    }
    if (document.queryCommandState('insertUnorderedList')) {
        items.push('unorderedList');
    }
    if (document.queryCommandState('justifyCenter')) {
        items.push('justifyCenter');
    }
    if (document.queryCommandState('justifyFull')) {
        items.push('justifyFull');
    }
    if (document.queryCommandState('justifyLeft')) {
        items.push('justifyLeft');
    }
    if (document.queryCommandState('justifyRight')) {
        items.push('justifyRight');
    }
    if (document.queryCommandState('insertHorizontalRule')) {
        items.push('horizontalRule');
    }
    var formatBlock = document.queryCommandValue('formatBlock');
    if (formatBlock.length > 0) {
        items.push(formatBlock);
    }

    window.android.stateCheck(encodeURI(items.join(',')));
}


RE.focus = function () {
    var range = document.createRange();
    range.selectNodeContents(RE.editor);
    range.collapse(false);
    var selection = window.getSelection();
    selection.removeAllRanges();
    selection.addRange(range);
    RE.editor.focus();
}

RE.blurFocus = function () {
    RE.editor.blur();
}



// Event Listeners
RE.editor.addEventListener("input", RE.callback);
RE.editor.addEventListener("keyup", function (e) {
    var KEY_LEFT = 37, KEY_UP = 38, KEY_RIGHT = 39, KEY_DOWN = 40;
    if (e.which == KEY_LEFT || e.which == KEY_UP || e.which == KEY_RIGHT || e.which == KEY_DOWN) {
        RE.enabledEditingItems(e);
    }
});
RE.editor.addEventListener("click", RE.enabledEditingItems);